# - * -coding: utf - 8 - * -
# The MIT License (MIT)

# Copyright (c) 2015 Stanislas Daniel Claude Dolcini Wildfiregames

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the Software), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, andor sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


import copy
import errno
import glob
import json
import os
import sys
import urllib2
import xml.etree.ElementTree as ET

reload(sys)
sys.setdefaultencoding('utf-8')

# ===================== PARAMS ======================
use_custom_xml_folder = False
# If set to false this script won't download the pictures for
# the units/buildings/techs so it should run without crashing.
# Warning: If there are no pictures LaTeX won't compile.
Is_there_an_Internet_connection = True
Revision = "18148"
InternetPathForIcons = "http://trac.wildfiregames.com/export/" + Revision \
					   + "/ps/trunk/binaries/data/mods/public/art/textures/ui/session/portraits/"
# ============ use True if there were custom changes to tex-files (which should be the case for final release)
Keep_Tex_Files = False
simulation_path = "D:/0AD/ps/trunk/binaries/data/mods/public/simulation/"
template_path = simulation_path + "templates/"
manual_generator_path ="D:/0AD/ps/trunk/source/tools/manualgenerator/"
save_path = manual_generator_path + "XML/"
tex_files_path = manual_generator_path + "tex-files/"

# Used to cache the dicts produced by get_template_as_dict
templates = {}
building_list = [[]]

def download_wiki_page(url):
	return

def convert_wiki_page():
	return

def download_portrait(civ, unit, stats):
	portrait = get_stat(stats, ["Identity", "Icon"])
	try:
		portrait_file = os.path.isfile(save_path + civ + '/' + unit + '/pic.png')
		if portrait_file is False:
			f = urllib2.urlopen(InternetPathForIcons + portrait)
			with open(save_path + civ + '/' + unit + '/pic.png', 'wb') as code:
				code.write(f.read())
	except:
		print "Image is missing"

## Generate a dictionnary for the template
# @param {string} template
# @return object containing the template or {}
def get_template_as_dict(template):
	# Not already cached -> retrieve
	if template not in templates:
		try:
			tree = ET.parse(get_template_file(template))
		except:
			tree = {};
			print("Error reading file \"" + template + ".xml\"")
			return {}
		root = tree.getroot()
		parent = root.get('parent')

		# Get values top-down (parent first)
		vals = {}
		if parent:
			# Get a deep copy, so we don't change the parent
			vals = copy.deepcopy(get_template_as_dict(parent))

		templates[template] = dict_temp_helper(root, vals)

	return templates[template]


def dict_temp_helper(element, vals={}):
	for el in list(element):
		if "disable" in el.attrib:
			# If this raises an exception, then the xml files
			# are doing something wrong.
			try:
				del vals[el.tag]
				continue
			except:
				print "Can't delete " + str(el.tag) + "From Values"
		if "datatype" in el.attrib and el.attrib["datatype"] == "tokens":
			if not el.tag in vals:
				vals[el.tag] = []
			# Merge the tokens (including removal)
			vals[el.tag].extend(dict_temp_helper(el, vals[el.tag]).split())
			# NOTE: We use sorted just to find all values that should be
			# removed (starting with '-'), but we don't want to change the
			# order
			for val in sorted(vals[el.tag]):
				# No more values to remove
				# TODO: Doesn't work with templates starting with
				#	   characters < '-', but we assume nobody will
				#	   feed this with such template names.
				if val[0] != '-':
					break
				# Remove the entry to be removed
				vals[el.tag].remove(val[1:])
				# Remove the remover
				vals[el.tag].remove(val)
		else:
			vals[el.tag] = dict_temp_helper(el, vals[el.tag] if el.tag in vals else {})
	if not vals or not isinstance(vals, dict):
		if element.text:
			return removeSuffix(element.text, ".0")
		elif "datatype" in element.attrib and element.attrib["datatype"] == "tokens":
			return ""
		else:
			# To support eg. <Looter/>
			return {}
	return vals


def generate_manual():
	units = get_unit_list()
	civs = get_civ_list()
	# animals = get_gaia_animals()

	#for civtemplates in animals:
	#	for animal in civtemplates:
	#		print_gaia_animals(animal)
	for civtemplates in units:
		for unit in civtemplates:
			print_unit(unit, civs)
	for civ in civs:
		for civilization in civ:
			print_civ(civilization, units)
	print_technology_list()


# Returns None in case the element specified by list isn't found.
def get_stat(stats, list):
	list.reverse()
	return get_stat_helper(stats, list)


def get_stat_helper(stats, list):
	val = stats.get(list.pop())
	if len(list) and val is not None:
		return get_stat_helper(val, list)
	return val


# Returns a list with the found elements or an empty list.
def get_stats(stats, list):
	list.reverse()
	retval = get_stats_helper(stats, list)
	return [] if retval is None else retval


def get_stats_helper(stats, l):
	key = l[-1]
	if key is None:
		ret = []
		if len(l[:-1]):
			for val in stats:
				b = get_stats_helper(stats[val], l[:-1])
				if b is not None:
					if isinstance(b, list):
						for a in b:
							ret.append(a)
					else:
						ret.append(b)
		else:
			if isinstance(stats, dict):
				for val in stats:
					ret.append(stats[val])
			else:
				ret = stats
		return ret
	else:
		val = stats.get(key)
		if len(l[:-1]) and val is not None:
			return get_stats_helper(val, l[:-1])
		return val


def get_template_file(template):
	if use_custom_xml_folder:
		# TODO: add the retrieval code here
		pass
	else:
		return template_path + template + '.xml'


# Return the entity's civ
def get_civ(stats):
	return get_stat(stats, ["Identity", "Civ"])


## Return a list that contains the civs names.
# @return {array} civList
def get_civ_list():
	civs = [os.listdir(simulation_path + 'data/civs')]
	for i in range(len(civs[0])):
		civs[0][i] = os.path.splitext(os.path.basename(civs[0][i]))[0]
	return civs


## Return a list containing the animals units.
# @return {array} units
def get_gaia_animals():
	units = [os.listdir(simulation_path + 'templates/gaia/')]
	unitsTemp = [[]]
	for i in range(len(units[0])):
		if "fauna" in units[0][i]:
			unitsTemp[0].append(units[0][i])
	units = unitsTemp
	for i in range(len(units[0])):
		units[0][i] = 'gaia/' + os.path.splitext(os.path.basename(units[0][i]))[0]
	return units


def print_technology_list():
	technologies = [os.listdir(tex_files_path + 'Technologies/')]
	with open("tex-files/technology.tex", "wb") as fp:
		for i in range(len(technologies[0])):
			fp.write("\input{tex-files/Technologies/" + str(technologies[0][i]) + "} \n")

##Loads the units into the array.
# Removes the rank variant, the house female citizen, and the barracks units.
# TODO : Only loads them if they cache is disabled.
def get_unit_list():
	units = [os.listdir(simulation_path + 'templates/units/')]
	unitsTemp = [[]]

	for i in range(len(units[0])):
		if "ptol" not in units[0][i] and "_barracks" not in units[0][i] and "_house" not in units[0][i] and "_a.xml" not in units[0][i] and "_e.xml" not in units[0][i] and "boudicca_sword" not in units[0][i]:
			unitsTemp[0].append(units[0][i])
#Ptolemaics have broken siege Engines
		if"ptol" in units[0][i] and "_packed" not in units[0][i] and "ptol" in units[0][i] and "_unpacked" not in units[0][i] and "_a.xml" not in units[0][i] and "_e.xml" not in units[0][i] and "_barracks" not in units[0][i] and "_house" not in units[0][i]:
			unitsTemp[0].append(units[0][i])

	units = unitsTemp
	for i in range(len(units[0])):
		units[0][i] = 'units/' + os.path.splitext(os.path.basename(units[0][i]))[0]
	return units


def make_sure_path_exists(path):
	try:
		os.makedirs(path)
	except OSError as exception:
		if exception.errno != errno.EEXIST:
			raise


## Generates the civ .tex file
# @param {string} civ
# @param {array} unitList
def print_civ(civ, units):
	with open(simulation_path + 'data/civs/' + civ + '.json') as data_file:
		data = json.load(data_file)
	CivName = data["Name"]
	# Open a file
	print "======== " + CivName + " ========="
	try:
		os.mkdir("./tex-files/" + civ)
	except:
		print 'Folder Already Exists'
	with open("tex-files/" + civ + "/" + civ + ".tex", "wb") as fp:
		for i in range(len(building_list[0])):
			if civ + "_" in building_list[0][i] and "_" + civ + "_" not in building_list[0][i] and "wallset" not in building_list[0][i]:
				fp.write("\input{tex-files/" + civ + "/" + building_list[0][i] + "} \n")
		for i in range(len(units[0])):
			if civ + "_" in units[0][i] and "_" + civ + "_" not in units[0][i]:
				fp.write("\input{tex-files/" + civ + "/" + units[0][i] + "} \n")


## Generates the cost .txt file
# @param stats
# @param {string} civ
# @param {string} unit
def print_cost(civ, unit, stats):
	Resources = get_stats(stats, ["Cost", "Resources"])
	# Remove some useless values
	# TODO: If this happens more than twice we should maybe make this a function, or part of get_stats
	# We assume furthermore that there is no tech/unit/structure that doesn't cost something.
	for res in list(Resources):
		if Resources[res] == "0":
			del Resources[res]
	# =======================Entity Build Time Cost==================================
	buildtime = get_stat(stats, ["Cost", "BuildTime"])
	if buildtime <= 0 or buildtime is None:
		buildtime = "0";
	# =======================Entity Population Cost==================================
	population = get_stat(stats, ["Cost", "Population"])
	if population is None or population == "0":
		write_file(civ, unit, "cost.txt", "Cost \>" +
				   ',\\,'.join(list('\\includegraphics[width=0.0225\\textwidth]{' + res + '.png}\\,' + Resources[res]
									for res in sorted(Resources, key=["food", "wood", "stone",
																	  "metal"].index))) + ',\\,\\includegraphics[width=0.0225\\textwidth]{buildtime.png}\\,' + buildtime)
	else:
		write_file(civ, unit, "cost.txt", "Cost \>" +
				   ',\\,'.join(list('\\includegraphics[width=0.0225\\textwidth]{' + res + '.png}\\,' + Resources[res]
									for res in sorted(Resources, key=["food", "wood", "stone",
																	  "metal"].index))) + ',\\,\\includegraphics[width=0.0225\\textwidth]{buildtime.png}\\,' + buildtime + ',\\,\\includegraphics[width=0.0225\\textwidth]{population.png}\\,' + population)


## Generates the Animal *.tex file
# @param {string} animal
def print_gaia_animals(animal):
	# Get Name Section
	stats = get_template_as_dict(animal)
	civ = "gaia"
	SpecificName = get_stat(stats, ["Identity", "SpecificName"])
	if SpecificName is None or SpecificName is "None":
		write_file(civ, animal, "spec_name.txt", "SpecificName \> IS MISSING!\\\\")
	else:
		write_file(civ, animal, "spec_name.txt", SpecificName)
	ResourceSupply = get_stat(stats, ["ResourceSupply", "Amount"])
	if ResourceSupply is None:
		write_file(civ, animal, "resource_supply.txt", "---")
	else:
		write_file(civ, animal, "resource_supply.txt", ResourceSupply)
	print_health(civ, animal, stats)
	# =======================Tex File Writing ============================

	if Keep_Tex_Files is False:
		with open(tex_files_path, "r") as f:
			get_tex_example = f.read()
		get_tex_output_file = get_tex_example.replace("EXAMPLE", civ + '/' + animal)
		make_sure_path_exists('tex-files/' + civ)
		try:
			with open('tex-files/' + animal + '.tex', "wb") as f:
				f.write(get_tex_output_file)
		except:
			print "Can't write " + 'tex-files/' + animal + '.tex'


## Generates the generic name *.txt file
# @param stats
# @param {string} civ
# @param {string} unit
def print_generic_name(stats, civ, unit):
	GenericName = get_stat(stats, ["Identity", "GenericName"])
	if GenericName is None:
		write_file(civ, unit, "gen_name.txt", "GenericName IS MISSING!")
	else:
		write_file(civ, unit, "gen_name.txt", GenericName)
	return GenericName


## Generates the health *.txt file
# @param stats
# @param {string} civ
# @param {string} unit
def print_health(civ, unit, stats):
	try:
		write_file(civ, unit, "health.txt", get_stat(stats, ["Health", "Max"]) + " &")
	except:
		write_file(civ, unit, "health.txt", "---" + " &")


## Generates the history *.txt file
# @param stats
# @param {string} civ
# @param {string} unit
def print_history(civ, unit, stats):
	try:
		History = str(get_stat(stats, ["Identity", "History"]))
	except:
		History = ""
	if History is None or History == "{}":
		write_file(civ, unit, "history.txt", "History IS MISSING!")
		return ""
	else:
		History = History.replace("%", "\\%")
		write_file(civ, unit, "history.txt", "\\emph{" + History + "}")
		return History


## Generates the improvements *.txt file
# @param stats
# @param {string} civ
# @param {string} unit
def print_improvements(civ, unit, stats):
	unit_classes = get_stats(stats, ["Identity", "Classes"])
	with open(save_path + civ + '/' + unit + '/improvements.txt', "w") as h:
		h.write("Improvements \\>\\begin{tabular}[t]{@{}p{0.65\\textwidth}}")
	for filename in glob.glob(save_path + "Technologies/*/affects_temp.txt"):
		with open(filename) as f:
			affects_temp = f.read()
			if affects_temp in unit_classes:
				bla = filename
				bla = bla.rstrip("affects_temp.txt")
				bla = bla + "gen_name.txt"
				bla = bla.replace('\\', '/')
				with open(bla, "r") as g:
					blu = g.read()
					with open(save_path + civ + '/' + unit + '/improvements.txt', "a") as h:
						h.write(blu + ", ")
			# ==== problem e.g. CitizenSoldier Melee
			if " " in affects_temp:
				first_part_of_long_affects_name = affects_temp[:affects_temp.find(" ")]
				#print(first_part_of_long_affects_name)
				if affects_temp.find("\n") != -1:
					secnd_part_of_long_affects_name = affects_temp[
													  affects_temp.find(" ") + len(" "):affects_temp.find("\n")]
				else:
					secnd_part_of_long_affects_name = affects_temp[affects_temp.find(" ") + len(" "):]
				#print(secnd_part_of_long_affects_name)
				if (first_part_of_long_affects_name in unit_classes) and (	secnd_part_of_long_affects_name in unit_classes):
					bla = filename
					bla = bla.rstrip("affects_temp.txt")
					bla = bla + "gen_name.txt"
					bla = bla.replace('\\', '/')
					with open(bla,"r+") as g:
						blu = g.read()
						with open(save_path + civ + '/' + unit + '/improvements.txt', "a") as h:
							h.write(blu + ", ")
	with open(save_path + civ + '/' + unit + '/improvements.txt', "r") as f:
		bla = f.read()
		if bla.endswith(", "):
			bla = bla.rstrip(", ")
			with open(save_path + civ + '/' + unit + '/improvements.txt', "w") as f:
				f.write(bla)
	with open(save_path + civ + '/' + unit + '/improvements.txt', "a+") as f:
		f.write("\\end{tabular}\\\\")
	with open(save_path + civ + '/' + unit + '/improvements.txt', "r") as f:
		if f.read().endswith("Improvements \\>\\begin{tabular}[t]{@{}p{0.65\\textwidth}}\\end{tabular}\\\\"):
			with open(save_path + civ + '/' + unit + '/improvements.txt', "w") as f:
				f.write("%")


## Generates the specific name *.txt file
# @param stats
# @param {string} civ
# @param {string} GenericName
# @param {string} unit
def print_specific_name(GenericName, stats, civ, unit):
	SpecificName = get_stat(stats, ["Identity", "SpecificName"])
	if SpecificName is None:
		write_file(civ, unit, "spec_name.txt", "SpecificName \> IS MISSING!\\\\")
		return ""
	else:
		# ================ as we don't want to have the name of the hero mentioned again
		if not (("Fauna" in GenericName) or (unit.find("hero") != -1) or ("Embassy" in GenericName) or ("Special Building" in GenericName)):
			if "ī́" in SpecificName:
				SpecificName = SpecificName.replace("ī́", "$\\acute{\\bar{\\textrm{\i}}}$")
			if "ḗ" in SpecificName:
				SpecificName = SpecificName.replace("ḗ", "$\\acute{\\bar{\\textrm{e}}}$")
			if "ā́" in SpecificName:
				SpecificName = SpecificName.replace("ā́", "$\\acute{\\bar{\\textrm{a}}}$")
			if "ṓ" in SpecificName:
				SpecificName = SpecificName.replace("ṓ", "$\\acute{\\bar{\\textrm{o}}}$")
			write_file(civ, unit, "spec_name.txt", "Specific Name \>" + SpecificName + "\\\\")
		else:
			write_file(civ, unit, "spec_name.txt", "%")
	return SpecificName


## Generates the specific name *.txt file
# @param stats
# @param {string} civ
# @param {string} unit
def print_tooltip(civ, unit, stats):
	Tooltip = get_stat(stats, ["Identity", "Tooltip"])
	if Tooltip is None:
		write_file(civ, unit, "tooltip.txt", "Tooltip IS MISSING!")
	else:
		write_file(civ, unit, "tooltip.txt", get_stat(stats, ["Identity", "Tooltip"]))


## Generates the unit .tex file
# @param {string} unit
# @param {string} civList
def print_unit(unit, civList):
	max_number_of_promotions = 2
	stats = get_template_as_dict(unit)
	damage_types = ["Crush", "Pierce", "Hack"]

	civ = get_civ(stats);
	if unit is None or civList is None:
		return

	# We don't check for units without factions.

	if civ not in civList[0]:
		print "Ignored: " + unit
		return

	print("========= " + unit + " =========")
	if "wallset" in unit:
		print_unit(get_stat(stats, ["WallSet", "Templates", "Gate"]), civList)
		print_unit(get_stat(stats, ["WallSet", "Templates", "WallLong"]), civList)
		unit = get_stat(stats, ["WallSet", "Templates", "Tower"])
		stats = get_template_as_dict(unit)

	# =======================Entity Generic Name==================================
	GenericName = print_generic_name(stats, civ, unit)
	# =======================Entity Specific Name==================================
	SpecificName = print_specific_name(GenericName, stats, civ, unit)
	# =======================Entity History==================================
	History = print_history(civ, unit, stats)
	# =======================Entity Health==================================
	Health = print_health(civ, unit, stats)
	# =======================Entity Tooltip==================================
	Tooltip = print_tooltip(civ, unit, stats)
	# =======================Entity Resources Cost==================================
	print_cost(civ, unit, stats)
	# =======================Entity Classes==================================
	print_improvements(civ, unit, stats)
	# =======================Entity Bonus Malus==================================
	print_bonus_malus(civ, unit, stats)
	# =======================Entity Portrait==================================
	if Is_there_an_Internet_connection is True:
		download_portrait(civ, unit, stats)
	# =======================Entity Velocity==================================
	print_velocity(civ, unit, stats)
	# =======================Entity Range==================================
	write_file(civ, unit, "range.txt", "--- &")
	# =======================Entity Armour==================================
	try:
		with open(save_path + civ + '/' + unit + '/armour.txt', "w") as f:
			f.write("")
	except:
		print "Unit is undefined"
	
	for i in range(0,len(damage_types)):
		armour = get_stats(stats, ["Armour", damage_types[i]])
		try:
			with open(save_path + civ + '/' + unit + '/armour.txt', "a") as f:
				f.write("\\includegraphics[width=0.0225\\textwidth]{" + damage_types[i] + ".png}\\," + armour + ",\\,")
		except:
			print "Can't open " + save_path + civ + '/' + unit + '/armour.txt'
	try:
		with open(save_path + civ + '/' + unit + '/armour.txt', "r") as f:
			bla = f.read()
			if bla.endswith(",\\,"):
				bla = removeSuffix(bla, ",\\,")
				with open(save_path + civ + '/' + unit + '/armour.txt', "w") as f:
					f.write(bla + " &")
	except:
		print "Can't open " + save_path + civ + '/' + unit + '/armour.txt'
	try:
		with open(save_path + civ + '/' + unit + '/attack.txt', "w") as f:
			f.write("--- &")
	except:
		print "Can't open " + save_path + civ + '/' + unit + '/attack.txt'
	b = ["Melee", "Ranged"]
	for i in range(0,len(b)):
		for j in range(0,len(damage_types)):
			attack = get_stat(stats, ["Attack", b[i], damage_types[j]])
			if attack != "0" and attack is not None:
				with open(save_path + civ + '/' + unit + '/attack.txt', "r") as f:
					if f.read().startswith("---"):
						with open(save_path + civ + '/' + unit + '/attack.txt', "w") as f:
							f.write("")
				with open(save_path + civ + '/' + unit + '/attack.txt', "a") as f:
					f.write("\\includegraphics[width=0.0225\\textwidth]{" + damage_types[j] + ".png}\\," + attack + ",\\,")
		if get_stat(stats, ["Attack", b[i], "MaxRange"]) is not None:
			if get_stat(stats, ["Attack", "Ranged", "MaxRange"]) is not None:
				with open(save_path + civ + '/' + unit + '/range.txt', "w") as f:
					f.write(get_stat(stats, ["Attack", b[i], "MaxRange"]) + " &")
			else:
				with open(save_path + civ + '/' + unit + '/range.txt', "w") as f:
					f.write(get_stat(stats, ["Attack", b[i], "MaxRange"]) + " &")

	if unit is None or civ is None or save_path is None:
		return
	with open(save_path + civ + '/' + unit + '/attack.txt', "r") as f:
		bla = f.read()
		if bla.endswith(",\\,"):
			bla = removeSuffix(bla, ",\\,")
			with open(save_path + civ + '/' + unit + '/attack.txt', "w") as f:
				f.write(bla + " &")

	for i in range(1,3):
		write_file(civ, unit, "attack" + str(i) + ".txt", "%")
		write_file(civ, unit, "armour" + str(i) + ".txt", "%")
		write_file(civ, unit, "speed" + str(i) + ".txt", "%")
		write_file(civ, unit, "range" + str(i) + ".txt", "%")
		write_file(civ, unit, "health" + str(i) + ".txt", "%")

	promotions = 0
	promotion_stats = stats
	while True:
		promotion = get_stat(promotion_stats, ["Promotion", "Entity"])
		if promotion is None:
			break
		promotions += 1
		promotion_str = str(promotions)
		promotion_stats = get_template_as_dict(promotion)
		health = get_stat(promotion_stats, ["Health", "Max"])
		armour = get_stats(promotion_stats, ["Armour"])

		if promotions == 1:
			with open(simulation_path + "/data/technologies/advanced_unit_bonus.json") as data_file:
				data = json.load(data_file)
				health = float(health) * data["modifications"][0]["multiply"]
				write_file(civ, unit, "health" + promotion_str + ".txt", "Advanced" + " & " + str(health) + " &")
				for i in range (1,4):
					armour[damage_types[i-1]] = str(int(armour[damage_types[i-1]]) + data["modifications"][i]["add"])
				write_file(civ, unit, "armour" + promotion_str + ".txt",',\\,'.join(list('\\includegraphics[width=0.0225\\textwidth]{' + a.lower() + '.png}\\,' + armour[a]for a in armour)) + ' &')
		if promotions == 2:
			with open(simulation_path + "/data/technologies/advanced_unit_bonus.json") as data_file:
				data = json.load(data_file)
				health = float(health) * data["modifications"][0]["multiply"]
			with open(simulation_path + "/data/technologies/elite_unit_bonus.json") as data_file:
				data = json.load(data_file)
				health = float(health) * data["modifications"][0]["multiply"]
				write_file(civ, unit, "health" + promotion_str + ".txt", "Elite" + " & " + str(health) + " &")
				for i in range (1,4):
					armour[damage_types[i-1]] = str(int(armour[damage_types[i-1]]) + data["modifications"][i]["add"])
				write_file(civ, unit, "armour" + promotion_str + ".txt",',\\,'.join(list('\\includegraphics[width=0.0225\\textwidth]{' + a.lower() + '.png}\\,' + armour[a]for a in armour)) + ' &')
		with open(save_path + civ + '/' + unit + '/range.txt', "r") as f:
			write_file(civ, unit, "range" + promotion_str + ".txt", f.read())

		# NOTE: No special handling for packed siege here, as we
		# don't have promotions for any packable units currently.
		write_file(civ, unit, "speed" + promotion_str + ".txt", get_stat(promotion_stats, ["UnitMotion", "WalkSpeed"]) + "/" + get_stat(promotion_stats,["UnitMotion", "Run","Speed"]) + "\\\\")

		with open(save_path + civ + '/' + unit + '/attack' + promotion_str + '.txt', "w") as f:
			f.write("--- &")
		b = ["Melee", "Ranged"]
		for i in range(0, len(b)):
			j = 0
			while j < len(damage_types):
				attack = get_stat(promotion_stats, ["Attack", b[i], damage_types[j]])
				if attack != "0" and attack is not None:
					with open(save_path + civ + '/' + unit + '/attack' + promotion_str + '.txt', "r") as f:
						if f.read().startswith("---"):
							with open(save_path + civ + '/' + unit + '/attack' + promotion_str + '.txt', "w") as f:
								f.write("")
					with open(save_path + civ + '/' + unit + '/attack' + promotion_str + '.txt', "a") as f:
						f.write("\\includegraphics[width=0.0225\\textwidth]{" + damage_types[j] + ".png}\\," + attack + ",\\,")
				j += 1
				if get_stat(stats, ["Attack", b[i], "MaxRange"]) is not None:
					if get_stat(stats, ["Attack", "Ranged", "MaxRange"]) is not None:
						with open(save_path + civ + '/' + unit + '/range.txt', "w") as f:
							f.write(get_stat(stats, ["Attack", b[i], "MaxRange"]) + " &")
					else:
						with open(save_path + civ + '/' + unit + '/range.txt', "w") as f:
							f.write(get_stat(stats, ["Attack", b[i], "MaxRange"]) + " &")
		with open(save_path + civ + '/' + unit + '/attack' + promotion_str + '.txt', "r") as f:
			bla = f.read()
			if bla.endswith(",\\,"):
				bla = removeSuffix(bla, ",\\,")
				with open(save_path + civ + '/' + unit + '/attack' + promotion_str + '.txt', "w") as f:
					f.write(bla + ' &')

	# Check if there are promotions and remove the \\ at the output of speed(#highest_promotion)
	i = max_number_of_promotions
	while i > -1:
		with open(save_path + civ + '/' + unit + '/speed' + str(i) + '.txt', "r") as f:
			input_file = f.read()
			if input_file.endswith("%"):
				i -= 1
				continue
			else:
				output = removeSuffix(input_file, "\\\\")
				with open(save_path + civ + '/' + unit + '/speed' + str(i) + '.txt', "w") as f:
					f.write(output)
				break

	buildable = get_stats(stats, ["Builder", "Entities"])
	if buildable is not None:
		for i in range(len(buildable)):
			buildable[i] = buildable[i].replace("{civ}", civ)
			if buildable[i] not in building_list[0]:
				print_unit(buildable[i], civList)
				building_list[0].append(buildable[i])



	Prod_Queue = os.path.isfile(save_path + civ + '/' + unit + '/ProductionQueue.txt')
	if Prod_Queue is False:
		write_file(civ, unit, "ProductionQueue.txt", "%")
	if Prod_Queue is True:
		os.remove(save_path + civ + '/' + unit + '/ProductionQueue.txt')

	production_queue = get_stats(stats, ["ProductionQueue", "Entities"])
	if production_queue is not None:
		with open(save_path + civ + '/' + unit + '/ProductionQueue.txt', "w+") as f:
			f.write("Trains ")
		for i in range(len(production_queue)):
			production_queue[i] = production_queue[i].replace("{civ}", civ)
			# print(production_queue[i])
			Prod_Queue_Temp = os.path.isfile(save_path + civ + '/' + unit + '/ProductionQueue_temp.txt')
			if Prod_Queue_Temp is False:
				production_queue = [word.replace('{civ}', civ) for word in production_queue]
				production_queue = [word.replace('_a\n', '_b\n') for word in
									production_queue]  # ====== to get the information for veteran units
				with open(save_path + civ + '/' + unit + '/ProductionQueue_temp.txt', "w") as f:
					f.write("\n".join(production_queue))
			# print('bla' + production_queue[i])
			prod_queue_stats = get_template_as_dict(production_queue[i])
			# print('blu' + str(prod_queue_stats))
			GenericName = get_stat(prod_queue_stats, ["Identity", "GenericName"])
			if GenericName is not None:
			 # ================ as we don't want to have "Fauna" or "Hero", but "sheep" and e.g. "Scipio"
				if (GenericName == "Fauna" or "Hero" in GenericName): 
					with open(save_path + civ + '/' + unit + '/ProductionQueue.txt', "a") as f:
						f.write("\\>" + get_stat(prod_queue_stats, ["Identity", "SpecificName"]) + "\\\\\n")
				else:
					with open(save_path + civ + '/' + unit + '/ProductionQueue.txt', "a") as f:
						f.write("\\>" + get_stat(prod_queue_stats, ["Identity", "GenericName"]) + "\\\\\n")

	with open(save_path + civ + '/' + unit + '/ProductionQueue.txt', "r") as f:
		if f.read().endswith("Trains "):
			with open(save_path + civ + '/' + unit + '/ProductionQueue.txt', "w") as f:
				f.write("%")

				# =======================Entity Technologies==================================

	i = 0
	Technologies_Raw = get_stats(stats, ["ProductionQueue", "Technologies"])
	if Technologies_Raw is not None:
		for i in range(len(Technologies_Raw)):
			if Technologies_Raw[i].startswith("pair_") is True:
				with open(simulation_path + 'data/technologies/' + Technologies_Raw[i] + '.json') as data_file:
					data = json.load(data_file)
					if "top" in data:
						Technologies_Raw.append(data["top"])
					if "bottom" in data:
						Technologies_Raw.append(data["bottom"])

			if not Technologies_Raw[i].startswith("pair"):
				with open(simulation_path + 'data/technologies/' + Technologies_Raw[i] + '.json') as data_file:
					data = json.load(data_file)

				make_sure_path_exists(save_path + "Technologies/" + Technologies_Raw[i])

				with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "gen_name.txt", "w") as f:
					if "genericName" in data:
						f.write(data["genericName"])
					else:
						f.write("%")

				with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "descript.txt", "w") as f:
					if "description" in data:
						f.write("\\emph{" + data["description"] + "}")
					else:
						f.write("%")

				with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "require.txt", "w") as f:
					if "requirementsTooltip" in data:
						bla = data["requirementsTooltip"]
						f.write(
							"Requirements \\>\\begin{tabular}[t]{@{}p{0.65\\textwidth}}" + bla + "\\end{tabular}\\\\")
					else:
						f.write("%")

				with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "tooltip.txt", "w") as f:
					if "tooltip" in data:
						bla = data["tooltip"]
						bla = bla.replace("%", "\\%")
						f.write(bla)
					else:
						f.write("%")

				with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "portrait.txt", "w") as f:
					if "icon" in data:
						f.write(data["icon"])
					else:
						f.write("%")

				if Is_there_an_Internet_connection is True:
					if "icon" in data:
						portrait = data["icon"]
						PortraitFile = os.path.isfile(save_path + "Technologies/" + Technologies_Raw[i] + '/pic.png')
						if PortraitFile is False:
							print "Downloading: " + InternetPathForIcons + "technologies/" + portrait
							f = urllib2.urlopen(InternetPathForIcons + "technologies/" + portrait)
							with open(save_path + "Technologies/" + Technologies_Raw[i] + '/pic.png', 'wb') as code:
								code.write(f.read())

				Tech_affect_txt = os.path.isfile(
					save_path + "Technologies/" + Technologies_Raw[i] + "/" + "affects_temp.txt")
				if Tech_affect_txt is True:
					os.remove(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "affects_temp.txt")

				with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "affects_temp.txt", "w+") as f:
					if "affects" in data:
						f.write("\n".join(data["affects"]))
					else:
						if "modifications" in data:
							for k in range(0, len(data["modifications"])):
								if "affects" in data["modifications"][k]:
									f.write("\n".join(data["modifications"][k]["affects"]) + "\n")


						if "supersedes" in data:
							parent_technology = data["supersedes"]
							with open(
															simulation_path + 'data/technologies/' + parent_technology + '.json') as data_file_parent:
								data_parent = json.load(data_file_parent)
								if "affects" in data_parent:
									f.write("\n".join(data_parent["affects"]))
						else:
							f.write("%")

				if "researchTime" in data:
					value_researchTime = data["researchTime"]
				else:
					value_researchTime = "0"

				if "cost" in data:
					if "food" in data["cost"]:
						value_food = data["cost"]["food"]
					else:
						value_food = "0"

					if "wood" in data["cost"]:
						value_wood = data["cost"]["wood"]
					else:
						value_wood = "0"

					if "stone" in data["cost"]:
						value_stone = data["cost"]["stone"]
					else:
						value_stone = "0"

					if "metal" in data["cost"]:
						value_metal = data["cost"]["metal"]
					else:
						value_metal = "0"
				else:
					value_food = "0"
					value_wood = "0"
					value_stone = "0"
					value_metal = "0"

				if value_food > 0:
					value_food = "\\includegraphics[width=0.0225\\textwidth]{food.png}\\," + str(value_food) + ",\\,"
				else:
					value_food = ""

				if value_wood > 0:
					value_wood = "\\includegraphics[width=0.0225\\textwidth]{wood.png}\\," + str(value_wood) + ",\\,"
				else:
					value_wood = ""

				if value_stone > 0:
					value_stone = "\\includegraphics[width=0.0225\\textwidth]{stone.png}\\," + str(value_stone) + ",\\,"
				else:
					value_stone = ""

				if value_metal > 0:
					value_metal = "\\includegraphics[width=0.0225\\textwidth]{metal.png}\\," + str(value_metal) + ",\\,"
				else:
					value_metal = ""

				with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "cost.txt", "w") as f:
					if (value_researchTime == "0") and (value_food == "0") and (value_wood == "0") and (
								value_stone == "0") and (value_metal == "0"):
						f.write("%")
					else:
						f.write(
							"Cost \>" + value_food + value_wood + value_stone + value_metal + '\\includegraphics[width=0.0225\\textwidth]{buildtime.png}\\,' + str(
								value_researchTime))

			if Keep_Tex_Files is False:
				if not Technologies_Raw[i].startswith("pair"):
					with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "affects_temp.txt", "r") as g:
						bla = g.read()
						bla = bla.replace("\n", ", ")
						bla = bla.rstrip("%")
						bla = bla.rstrip(", ")
						with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "affects.txt", "w") as h:
							h.write(
								"Affects \\>\\begin{tabular}[t]{@{}p{0.65\\textwidth}}" + bla + "\\end{tabular}\\\\")
					with open("tex-files/example_tech.tex", "r") as f:
						get_tex_example_tech = f.read()
						get_tex_output_file_tech = get_tex_example_tech.replace("EXAMPLE","Technologies/" + Technologies_Raw[i])
					make_sure_path_exists('tex-files/Technologies/')
					make_sure_path_exists('tex-files/' + civ + '/units/')
					try:
						with open('tex-files/Technologies/' + Technologies_Raw[i] + '.tex', "wb") as f:
							f.write(get_tex_output_file_tech)
					except:
						print "Can't open " + Technologies_Raw[i]

		i += 1
		remove_double_entries_technology_raw = list(set(Technologies_Raw))
		with open(save_path + civ + '/' + unit + '/Technology_Temp.txt', "w") as f:
			f.write("\n".join(remove_double_entries_technology_raw))
	# ======================Production Queue===================================

	with open(save_path + civ + '/' + unit + '/trains_in.txt', "w") as f:
		f.write("Train in ")

	if unit.startswith("units"):
		for root, dirs, folder in os.walk(save_path):
			for files in folder:
				if files.endswith("ProductionQueue_temp.txt"):
					with open(os.path.join(root, files), "r") as f:
						input_file = f.read()
						is_this_unit_trainable = str(input_file.find(unit))
						# ==============================================
						# ======= Special handling for packed siege ====
						# ==============================================
						if unit.endswith("unpacked"):
							unit_siege_packed = removeSuffix(unit, "unpacked") + "packed"
							is_this_unit_trainable = str(input_file.find(unit_siege_packed))

						if is_this_unit_trainable != str(-1):
							with open(os.path.join(root) + '/gen_name.txt', "r") as f:
								can_train_unit = str(f.read())
								with open(save_path + civ + '/' + unit + '/trains_in.txt', "a") as f:
									f.write("\\>" + can_train_unit + "\\\\\n")

	with open(save_path + civ + '/' + unit + '/trains_in.txt', "r") as f:
		if f.read().endswith("Train in "):
			with open(save_path + civ + '/' + unit + '/trains_in.txt', "w") as f:
				f.write("%")
	if Keep_Tex_Files is False:
		with open(tex_files_path + "example.tex", "r") as f:
			get_tex_example = f.read()
		get_tex_output_file = get_tex_example.replace("EXAMPLE", civ + '/' + unit)
		make_sure_path_exists('tex-files/' + civ + '/structures/')
		make_sure_path_exists('tex-files/' + civ + '/units/')
		try:
			with open('tex-files/' + civ + '/' + unit + '.tex', "wb") as f:
				f.write(get_tex_output_file)
		except:
			print "Can't write " + 'tex-files/' + civ + '/' + unit + '.tex'


## Generates the velocity .txt file
# @param stats
# @param {string} civ
# @param {string} unit
def print_velocity(civ, unit, stats):
	# Packed siege special case
	if unit[-9:] == "_unpacked":
		packed_stats = get_template_as_dict(unit[:-9] + "_packed")
		Velocity_Walk = get_stat(packed_stats, ["UnitMotion", "WalkSpeed"])
		Velocity_Run = get_stat(packed_stats, ["UnitMotion", "Run", "Speed"])
	else:
		Velocity_Walk = get_stat(stats, ["UnitMotion", "WalkSpeed"])
		Velocity_Run = get_stat(stats, ["UnitMotion", "Run", "Speed"])

	if Velocity_Run is None:
		Velocity_Run = "---"
	if Velocity_Walk is None:
		Velocity_Walk = "---"
	if Velocity_Walk == "---" and Velocity_Run == "---":
		write_file(civ, unit, "speed0.txt", Velocity_Walk + '/' + Velocity_Run + '\\\\')
	else:
		write_file(civ, unit, "speed0.txt", Velocity_Walk + '/' + Velocity_Run + '\\\\')
	write_file(civ, unit, "range.txt", "--- &")


## Removes the suffix from a string
# @param {string} s the string
# @param {string} suffix
# @return s string without suffix
def removeSuffix(s, suffix):
	if s.endswith(suffix):
		return s[:-len(suffix)]
	return s


def write_file(civ, unit, file, val):
	try:
		path = save_path + civ + '/' + unit + '/'
		make_sure_path_exists(path)
		with open(path + file, 'wb') as f:
			f.write(val)
	except:
		print "Can't write file"


# NOTE: Don't use the first None as we only have Melee, Ranged, Charge and Slaughter defined
# and some bonuses are defined in the same way for Melee and Charge
# Just use Melee and Ranged (Slaughter doesn't support bonuses and Charge isn't implemented yet)
# TODO Stan : Capturing is there now.
def print_bonus_malus(civ, unit, stats):
	with open(save_path + civ + '/' + unit + '/bonus.txt', "wb") as f:
		f.write("Bonus vs.")
		f.write("\\\\")
	with open(save_path + civ + '/' + unit + '/malus.txt', "wb") as f:
		f.write("Malus vs.")
		f.write("\\\\")
	i = 0
	b = ["Melee", "Ranged", "Capture"]

	for i in range(0,len(b)):
		for a in get_stats(stats, ["Attack", b[i], "Bonuses", None]):
			make_sure_path_exists(save_path + civ + '/' + unit + '/')
			if float(a["Multiplier"]) > 1:
				bonus = "{}\n".format("\>" + (a["Classes"] if a["Classes"] != {} else a["Civ"]) + " ($\\times$" + a[
					"Multiplier"] + ")" + "\\\\")
				if bonus.startswith("\n"):
					bonus = bonus.lstrip('\n')
				if bonus.find("iber") is not None:
					bonus = bonus.replace("iber", "Iberians")
				if bonus.find("rome") is not None:
					bonus = bonus.replace("rome", "Romans")
				if bonus.find("celt") is not None:
					bonus = bonus.replace("celt", "Celts")
				if bonus.find("pers") is not None:
					bonus = bonus.replace("pers", "Persians")
				if bonus.find("cart") is not None:
					bonus = bonus.replace("cart", "Carthaginians")
				with open(save_path + civ + '/' + unit + '/bonus.txt', "a") as f:
					f.write(bonus)
			if float(a["Multiplier"]) < 1:
				malus = "{}\n".format("\>" + (a["Classes"] if a["Classes"] != {} else a["Civ"]) + " ($\\times$" + a[
					"Multiplier"] + ")" + "\\\\")
				if malus.startswith("\n"):
					malus = malus.lstrip('\n')
				with open(save_path + civ + '/' + unit + '/malus.txt', "a") as f:
					f.write(malus)

	# i = 0
	b = ["bonus", "malus"]
	for i in range(0,len(b)):
		f = open(save_path + civ + '/' + unit + '/' + b[i] + '.txt', "r").read()
		if f.endswith(" vs. \n"):
			with open(save_path + civ + '/' + unit + '/' + b[i] + '.txt', "w") as f:
				f.write("%")


# Main Loop
generate_manual()
