#!/usr/bin/env python
import xml.etree.ElementTree as ET
import copy
import os
import errno
import urllib.request

import json

import glob
 
use_custom_xml_folder = False

Is_there_an_Internet_connection = True #======== if set to false this script won't download the pictures for the units/buildings/techs, so it should run without crashing. Warning: If there are no pictures LaTeX won't compile.
Revision = "13119"
InternetPathForIcons = "http://trac.wildfiregames.com/export/" + Revision + "/ps/trunk/binaries/data/mods/public/art/textures/ui/session/portraits/"

Keep_Tex_Files = False #============ use True if there were custom changes to tex-files (which should be the case for final release) 

template_path = "simulation/templates/"
 
save_path = "XML/"





 
# Used to cache the dicts produced by get_template_as_dict
templates = {}

def removeSuffix(s, suffix):
    if s.endswith(suffix):
        return s[:-len(suffix)]
    return s
 
def get_template_as_dict(template):
    # Not already cached -> retrieve
    if template not in templates:
        tree = ET.parse(get_template_file(template))
        if not tree:
            # TODO: Raise exception?
            print("Error reading file \"" + template + ".xml\"")
            return {}
        root = tree.getroot()
        parent = root.get('parent')
        
        # Get values top-down (parent first)
        vals = {}
        if parent:
            # Get a deep copy, so we don't change the parent
            vals = copy.deepcopy(get_template_as_dict(parent))
        
        templates[template] = dict_temp_helper(root, vals)
    
    return templates[template]
 
 
def dict_temp_helper(element, vals={}):
    for el in list(element):
        if "disable" in el.attrib: 
            # If this raises an exception, then the xml files
            # are doing something wrong.
            del vals[el.tag]
            continue
        if "datatype" in el.attrib and el.attrib["datatype"] == "tokens":
            if not el.tag in vals:
                vals[el.tag] = []
            # Merge the tokens (including removal)
            vals[el.tag].extend(dict_temp_helper(el, vals[el.tag]).split())
            # NOTE: We use sorted just to find all values that should be
            # removed (starting with '-'), but we don't want to change the
            # order
            for val in sorted(vals[el.tag]):
                # No more values to remove
                # TODO: Doesn't work with templates starting with
                #       characters < '-', but we assume nobody will
                #       feed this with such template names.
                if val[0] != '-':
                    break
                # Remove the entry to be removed
                vals[el.tag].remove(val[1:])
                # Remove the remover
                vals[el.tag].remove(val)
        else:
            vals[el.tag] = dict_temp_helper(el, vals[el.tag] if el.tag in vals else {})
    if not vals or not isinstance(vals, dict):
        if element.text:
            return removeSuffix(element.text, ".0")
        elif "datatype" in element.attrib and element.attrib["datatype"] == "tokens":
            return ""
        else:
            # To support eg. <Looter/>
            return {}    
    return vals
 
 
# Returns None in case the element specified by list isn't found
def get_stat(stats, list):
    list.reverse()
    return get_stat_helper(stats, list)
 
def get_stat_helper(stats, list):
    val = stats.get(list.pop())
    if len(list) and val is not None:
        return get_stat_helper(val, list)
    return val
 
 
# Returns a list with the found elements
# or None
def get_stats(stats, list):
    list.reverse()
    retval = get_stats_helper(stats, list)
    return [] if retval is None else retval


 
def get_stats_helper(stats, l):
    key = l[-1]
    if key is None:
        ret = []
        if len(l[:-1]):
            for val in stats:
                b = get_stats_helper(stats[val], l[:-1])
                if b is not None:
                    if isinstance(b, list):
                        for a in b:
                            ret.append(a)
                    else:
                        ret.append(b)
        else:
            if isinstance(stats, dict):
                for val in stats:
                    ret.append(stats[val])
            else:
                ret = stats
        return ret
    else:
        val = stats.get(key)
        if len(l[:-1]) and val is not None:
            return get_stats_helper(val, l[:-1])
        return val
 
 
def get_template_file(template):
    if use_custom_xml_folder:
        # TODO: add the retrieval code here
        pass
    else:
        return template_path + template + '.xml'
 
 
def make_sure_path_exists(path):
    try:
        os.makedirs(path)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise
 
def write_file(civ, unit, file, val):
    path = save_path + civ + '/' + unit + '/'
    make_sure_path_exists(path)
    f = open(path + file, 'w', encoding="utf-8")
    f.write(val)
    f.close()
 


 
def printUnit(unit):
    stats = get_template_as_dict(unit)
    
    print("=========")
    print(unit)

    if "wallset" in unit:
        printUnit(get_stat(stats, ["WallSet", "Templates", "Gate"]))
        printUnit(get_stat(stats, ["WallSet", "Templates", "WallLong"]))
        unit = get_stat(stats, ["WallSet", "Templates", "Tower"])
        stats = get_template_as_dict(unit)

        
        

#====================================================================================================================
#==================================== Civ + Specific Name + Generic Name + Resources section. Works. Note: ==========
#==================================== There is no \\ at the end of the resource-part, as we suppose it's the ========
#==================================== last item in the table of the TeX-file ========================================
#====================================================================================================================    
    civ = get_stat(stats, ["Identity", "Civ"])
    print(civ)

    GenericName = get_stat(stats, ["Identity", "GenericName"])
    if GenericName is None:
        write_file(civ, unit, "gen_name.txt", "GenericName IS MISSING!")
    else:
        if ("Fauna" in GenericName) or (unit.find("hero") != -1) or ("Embassy" in GenericName) or ("Special Building" in GenericName): #================ as we don't want to have "Fauna" or "Hero", but "sheep" and e.g. "Scipio"
            SpecificName = get_stat(stats, ["Identity", "SpecificName"])
            if SpecificName is None:
                write_file(civ, unit, "spec_name.txt", "SpecificName \> IS MISSING!\\\\")
            else:
                if "ī́" in SpecificName:
                    SpecificName = SpecificName.replace("ī́", "$\\acute{\\bar{\\textrm{\i}}}$")
                if "ḗ" in SpecificName:
                    SpecificName = SpecificName.replace("ḗ", "$\\acute{\\bar{\\textrm{e}}}$")
                if "ā́" in SpecificName:
                    SpecificName = SpecificName.replace("ā́", "$\\acute{\\bar{\\textrm{a}}}$")
                if "ṓ" in SpecificName:
                    SpecificName = SpecificName.replace("ṓ", "$\\acute{\\bar{\\textrm{o}}}$")
            write_file(civ, unit, "gen_name.txt", SpecificName)
        else:
            write_file(civ, unit, "gen_name.txt", get_stat(stats, ["Identity", "GenericName"]))
    
    SpecificName = get_stat(stats, ["Identity", "SpecificName"])
    if SpecificName is None:
        write_file(civ, unit, "spec_name.txt", "SpecificName \> IS MISSING!\\\\")
    else:
        if not (("Fauna" in GenericName) or (unit.find("hero") != -1) or ("Embassy" in GenericName) or ("Special Building" in GenericName)): #================ as we don't want to have the name of the hero mentioned again
            if "ī́" in SpecificName:
                SpecificName = SpecificName.replace("ī́", "$\\acute{\\bar{\\textrm{\i}}}$")
            if "ḗ" in SpecificName:
                SpecificName = SpecificName.replace("ḗ", "$\\acute{\\bar{\\textrm{e}}}$")
            if "ā́" in SpecificName:
                SpecificName = SpecificName.replace("ā́", "$\\acute{\\bar{\\textrm{a}}}$")
            if "ṓ" in SpecificName:
                SpecificName = SpecificName.replace("ṓ", "$\\acute{\\bar{\\textrm{o}}}$")
            write_file(civ, unit, "spec_name.txt", "Specific Name \>" + SpecificName + "\\\\")
        else:
            write_file(civ, unit, "spec_name.txt", "%")
 
 
    resources = get_stats(stats, ["Cost", "Resources"])
    buildtime = get_stat(stats, ["Cost", "BuildTime"])
    population = get_stat(stats, ["Cost", "Population"])
    # Remove some useless values
    # TODO: If this happens more than twice we should maybe make this a function, or part of get_stats
    # Note: If there should ever be another entry after Cost \> ... in LaTeX, one has to add \\\\ at the end of the Cost.txt output.
    # We assume furthermore that there is no tech/unit/structure that doesn't cost something.
    for res in list(resources):
        if resources[res] == "0":
            del resources[res]
    if population is None or population == "0":
        write_file(civ, unit, "cost.txt", "Cost \>" +
            ',\\,'.join(list('\\includegraphics[width=0.0225\\textwidth]{'+res+'.png}\\,'+resources[res]
            for res in sorted(resources, key=["food", "wood", "stone", "metal"].index)))+',\\,\\includegraphics[width=0.0225\\textwidth]{buildtime.png}\\,'+buildtime)
    else:
        write_file(civ, unit, "cost.txt", "Cost \>" +
            ',\\,'.join(list('\\includegraphics[width=0.0225\\textwidth]{'+res+'.png}\\,'+resources[res]
            for res in sorted(resources, key=["food", "wood", "stone", "metal"].index)))+',\\,\\includegraphics[width=0.0225\\textwidth]{buildtime.png}\\,'+buildtime+',\\,\\includegraphics[width=0.0225\\textwidth]{population.png}\\,'+population)
#====================================================================================================================
#==================================== Bonus and Malus section. Works. ===============================================
#====================================================================================================================
    # NOTE: Don't use the first None as we only have Melee, Ranged, Charge and Slaughter defined
    # and some bonuses are defined in the same way for Melee and Charge
    # Just use Melee and Ranged (Slaughter doesn't support bonuses and Charge isn't implemented yet)
    with open(save_path + civ + '/' + unit + '/bonus.txt', "w", encoding="utf-8") as f:
        f.write("Bonus vs. \n")
    with open(save_path + civ + '/' + unit + '/malus.txt', "w", encoding="utf-8") as f:
        f.write("Malus vs. \n")

    i = 0
    b = ["Melee", "Ranged"]
    while i < 2:  
        for a in get_stats(stats, ["Attack", b[i], "Bonuses", None]):
            make_sure_path_exists(save_path + civ + '/' + unit + '/')        
            if float(a["Multiplier"]) > 1:
                bonus = "{}\n".format("\>" + (a["Classes"] if a["Classes"] != {} else a["Civ"]) + " ($\\times$" + a["Multiplier"] + ")" + "\\\\")
                if bonus.startswith("\n"):
                    bonus = bonus.lstrip('\n')
                if bonus.find("iber") is not None:
                    bonus = bonus.replace("iber", "Iberians")
                if bonus.find("rome") is not None:
                    bonus = bonus.replace("rome", "Romans")
                if bonus.find("celt") is not None:
                    bonus = bonus.replace("celt", "Celts")
                if bonus.find("pers") is not None:
                    bonus = bonus.replace("pers", "Persians")
                if bonus.find("cart") is not None:
                    bonus = bonus.replace("cart", "Carthaginians")
                with open(save_path + civ + '/' + unit + '/bonus.txt', "a", encoding="utf-8") as f:
                    f.write(bonus)  
            if float(a["Multiplier"]) < 1:
                malus = "{}\n".format("\>" + (a["Classes"] if a["Classes"] != {} else a["Civ"]) + " ($\\times$" + a["Multiplier"] + ")" + "\\\\")
                if malus.startswith("\n"):
                    malus = malus.lstrip('\n')
                with open(save_path + civ + '/' + unit + '/malus.txt', "a", encoding="utf-8") as f:
                    f.write(malus)
        i += 1

    i = 0
    b = ["bonus", "malus"]
    while i < 2:
        f = open(save_path + civ + '/' + unit + '/' + b[i] + '.txt', "r", encoding="utf-8").read()
        if f.endswith(" vs. \n"):
            with open(save_path + civ + '/' + unit + '/' + b[i] + '.txt', "w", encoding="utf-8") as f:
                f.write("%")
        i += 1
#====================================================================================================================    

    if Is_there_an_Internet_connection is True:
        portrait = get_stat(stats, ["Identity", "Icon"])
        PortraitFile = os.path.isfile(save_path + civ + '/' + unit + '/pic.png')
        if PortraitFile is False:
            f = urllib.request.urlopen(InternetPathForIcons + portrait)
            with open(save_path + civ + '/' + unit + '/pic.png', 'wb') as code:
                code.write(f.read())



            
    History = str(get_stat(stats, ["Identity", "History"]))
    if History is None or History == "{}":
        write_file(civ, unit, "history.txt", "History IS MISSING!")
    else:
        History = History.replace("%", "\\%")
        write_file(civ, unit, "history.txt", "\\emph{" + History + "}")

   
    Tooltip = get_stat(stats, ["Identity", "Tooltip"])
    if Tooltip is None:
        write_file(civ, unit, "tooltip.txt", "Tooltip IS MISSING!")
    else:
        write_file(civ, unit, "tooltip.txt", get_stat(stats, ["Identity", "Tooltip"]))
    

    write_file(civ, unit, "health.txt", get_stat(stats, ["Health", "Max"]) + " &")


    

    #Packed siege special case
    if unit[-9:] == "_unpacked":
        packed_stats = get_template_as_dict(unit[:-9]+"_packed")
        Velocity_Walk = get_stat(packed_stats, ["UnitMotion", "WalkSpeed"])      
        Velocity_Run = get_stat(packed_stats, ["UnitMotion", "Run", "Speed"])
        if Velocity_Run is None:
            Velocity_Run = "---"
    else:
        Velocity_Walk = get_stat(stats, ["UnitMotion", "WalkSpeed"])      
        Velocity_Run = get_stat(stats, ["UnitMotion", "Run", "Speed"])
        if Velocity_Run is None:
            Velocity_Run = "---"
        if Velocity_Walk is None:
            Velocity_Walk = "---"

    write_file(civ, unit, "speed0.txt", Velocity_Walk + '/' + Velocity_Run + '\\\\')
    if Velocity_Walk == "---" and Velocity_Run == "---":
        write_file(civ, unit, "speed0.txt", "---\\\\")
    write_file(civ, unit, "range.txt", "--- &")
        
    
    i = 0
    b = ["Hack", "Pierce", "Crush"]
    with open(save_path + civ + '/' + unit + '/armour.txt', "w", encoding="utf-8") as f:
            f.write("")
    while i < 3:
        armour = get_stats(stats, ["Armour", b[i]])
        with open(save_path + civ + '/' + unit + '/armour.txt', "a", encoding="utf-8") as f:
            f.write("\\includegraphics[width=0.0225\\textwidth]{"+b[i]+".png}\\,"+armour+",\\,")
        i += 1
    with open(save_path + civ + '/' + unit + '/armour.txt', "r", encoding="utf-8") as f:
        bla = f.read()
        if bla.endswith(",\\,"):
            bla = removeSuffix(bla, ",\\,")
            with open(save_path + civ + '/' + unit + '/armour.txt', "w", encoding="utf-8") as f:
                f.write(bla + " &")



    with open(save_path + civ + '/' + unit + '/attack.txt', "w", encoding="utf-8") as f:
        f.write("--- &")
    i = 0
    b = ["Melee", "Ranged"]
    while i < 2:
        j = 0
        c = ["Hack", "Pierce", "Crush"]
        while j < 3:
            attack = get_stat(stats, ["Attack", b[i], c[j]])
            if attack != "0" and attack is not None:
                with open(save_path + civ + '/' + unit + '/attack.txt', "r", encoding="utf-8") as f:
                    if f.read().startswith("---"):
                        with open(save_path + civ + '/' + unit + '/attack.txt', "w", encoding="utf-8") as f:
                            f.write("")
                with open(save_path + civ + '/' + unit + '/attack.txt', "a", encoding="utf-8") as f:
                    f.write("\\includegraphics[width=0.0225\\textwidth]{"+c[j]+".png}\\,"+attack+",\\,")             
            j += 1       
        if get_stat(stats, ["Attack", b[i], "MaxRange"]) is not None:
            if get_stat(stats, ["Attack", "Ranged", "MaxRange"]) is not None:
                with open(save_path + civ + '/' + unit + '/range.txt', "w", encoding="utf-8") as f:
                        f.write(get_stat(stats, ["Attack", b[i], "MaxRange"]) + " &")
            else:
                with open(save_path + civ + '/' + unit + '/range.txt', "w", encoding="utf-8") as f:
                        f.write(get_stat(stats, ["Attack", b[i], "MaxRange"]) + " &")
        i += 1
    with open(save_path + civ + '/' + unit + '/attack.txt', "r", encoding="utf-8") as f:
        bla = f.read()
        if bla.endswith(",\\,"):
            bla = removeSuffix(bla, ",\\,")
            with open(save_path + civ + '/' + unit + '/attack.txt', "w", encoding="utf-8") as f:
                f.write(bla + " &")


    i = 1
    while i < 3:
        write_file(civ, unit, "attack" + str(i) + ".txt", "%")
        write_file(civ, unit, "armour" + str(i) + ".txt", "%")
        write_file(civ, unit, "speed" + str(i) + ".txt", "%")
        write_file(civ, unit, "range" + str(i) + ".txt", "%")
        write_file(civ, unit, "health" + str(i) + ".txt", "%")
        i += 1

    
    promotions = 0
    promotion_stats = stats
    while True:
        promotions += 1
        promotion_str = str(promotions)

        promotion = get_stat(promotion_stats, ["Promotion", "Entity"])
        #print("This should be the promotion2 " + str(promotion))
        if promotion is None:
            break
        promotion_stats = get_template_as_dict(promotion)
        #print("This should be the promotion2 " + str(promotion_stats))
        #print("Promotion "+str(promotions)+":")
        #print(promotion)
        #print("Health")
 
        
        health = get_stat(promotion_stats, ["Health", "Max"])
        #print(health)
        higher_rank = "Advanced"
        even_higher_rank = "Elite"
        if promotions == 1:
            write_file(civ, unit, "health" + promotion_str + ".txt", higher_rank + " & " + health + " &")
        if promotions == 2:
            write_file(civ, unit, "health" + promotion_str + ".txt", even_higher_rank + " & " + health + " &")

        with open(save_path + civ + '/' + unit + '/range.txt', "r", encoding="utf-8") as f:
            write_file(civ, unit, "range" + promotion_str + ".txt", f.read())
        
        
        
        
        # NOTE: No special handling for packed siege here, as we
        # don't have promotions for any packable units currently.
        write_file(civ, unit, "speed" + promotion_str + ".txt", get_stat(promotion_stats, ["UnitMotion", "WalkSpeed"]) + "/" + get_stat(promotion_stats, ["UnitMotion", "Run", "Speed"]) + "\\\\")

        

        armour = get_stats(promotion_stats, ["Armour"])      
        write_file(civ, unit, "armour" + promotion_str + ".txt",
            ',\\,'.join(list('\\includegraphics[width=0.0225\\textwidth]{'+a.lower()+'.png}\\,'+armour[a]
            for a in sorted(armour))) + ' &')

        with open(save_path + civ + '/' + unit + '/attack'+promotion_str+'.txt', "w", encoding="utf-8") as f:
            f.write("--- &")
        i = 0
        b = ["Melee", "Ranged"]
        while i < 2:
            j = 0
            c = ["Hack", "Pierce", "Crush"]
            while j < 3:
                attack = get_stat(promotion_stats, ["Attack", b[i], c[j]])
                if attack != "0" and attack is not None:
                    with open(save_path + civ + '/' + unit + '/attack'+promotion_str+'.txt', "r", encoding="utf-8") as f:
                        if f.read().startswith("---"):
                            with open(save_path + civ + '/' + unit + '/attack'+promotion_str+'.txt', "w", encoding="utf-8") as f:
                                f.write("")
                    with open(save_path + civ + '/' + unit + '/attack'+promotion_str+'.txt', "a", encoding="utf-8") as f:
                        f.write("\\includegraphics[width=0.0225\\textwidth]{"+c[j]+".png}\\,"+attack+",\\,")             
                j += 1
                if get_stat(stats, ["Attack", b[i], "MaxRange"]) is not None:
                    if get_stat(stats, ["Attack", "Ranged", "MaxRange"]) is not None:
                        with open(save_path + civ + '/' + unit + '/range.txt', "w", encoding="utf-8") as f:
                                f.write(get_stat(stats, ["Attack", b[i], "MaxRange"]) + " &")
                    else:
                        with open(save_path + civ + '/' + unit + '/range.txt', "w", encoding="utf-8") as f:
                                f.write(get_stat(stats, ["Attack", b[i], "MaxRange"]) + " &")
            i += 1
        with open(save_path + civ + '/' + unit + '/attack'+promotion_str+'.txt', "r", encoding="utf-8") as f:
            bla = f.read()
            if bla.endswith(",\\,"):
                bla = removeSuffix(bla, ",\\,")
                with open(save_path + civ + '/' + unit + '/attack'+promotion_str+'.txt', "w", encoding="utf-8") as f:
                    f.write(bla + ' &') 
    
    # TODO: Should we move the civ replacement somewhere else?
    # Currently a useless section, as the stuff units can build isn't needed anywhere.
    #buildable = get_stats(stats, ["Builder", "Entities"])
    #if buildable is not None:
    #    for i in range(len(buildable)):
    #        buildable[i] = buildable[i].replace("{civ}", civ)
    #    
    #    for building in buildable:
    #        building_stats = get_template_as_dict(building)
    #        #print(get_stat(building_stats, ["Identity", "GenericName"]))
    #        list_buildings = get_stat(building_stats, ["Identity", "GenericName"])
    #       
    #        list_buildings_format = "{}\n".format(list_buildings)
    #        print(list_buildings_format)
    #        if list_buildings_format.startswith("\n"):
    #            list_buildings_format = list_buildings_format.lstrip('\n')
    #        
    #        make_sure_path_exists(save_path + civ + '/' + unit + '/')
    #        with open(save_path + civ + '/' + unit + '/Builds.txt', "a", encoding="utf-8") as f:
    #            f.write(list_buildings_format)


    # Check if there are promotions and remove the \\ at the output of speed(#highest_promotion)
    i = 2
    while i > -1:
        with open(save_path + civ + '/' + unit + '/speed' + str(i) +'.txt', "r", encoding="utf-8") as f:
            input_file = f.read()
            if input_file.endswith("%"):
                i -= 1
                continue
            else:
                output = removeSuffix(input_file, "\\\\")
                with open(save_path + civ + '/' + unit + '/speed' + str(i) +'.txt', "w", encoding="utf-8") as f:
                    f.write(output)
                break
        

    


    



    buildable = get_stats(stats, ["Builder", "Entities"])
    if buildable is not None:
        for i in range(len(buildable)):
            buildable[i] = buildable[i].replace("{civ}", civ)            
            printUnit(buildable[i])




    Prod_Queue = os.path.isfile(save_path + civ + '/' + unit + '/ProductionQueue.txt')
    if Prod_Queue is False:
        write_file(civ, unit, "ProductionQueue.txt", "%")
    if Prod_Queue is True:
        os.remove(save_path + civ + '/' + unit + '/ProductionQueue.txt')
   
    production_queue = get_stats(stats, ["ProductionQueue", "Entities"])
    if production_queue is not None:
        with open(save_path + civ + '/' + unit + '/ProductionQueue.txt', "w", encoding="utf-8") as f:
                f.write("Trains ")               
        for i in range(len(production_queue)):
            production_queue[i] = production_queue[i].replace("{civ}", civ)
            #print(production_queue[i])
            Prod_Queue_Temp = os.path.isfile(save_path + civ + '/' + unit + '/ProductionQueue_temp.txt')
            if Prod_Queue_Temp is False:
                production_queue = [word.replace('{civ}', civ) for word in production_queue]
                production_queue = [word.replace('_a\n', '_b\n') for word in production_queue] #====== to get the information for veteran units            
                with open(save_path + civ + '/' + unit + '/ProductionQueue_temp.txt', "w", encoding="utf-8") as f:
                    f.write("\n".join(production_queue))
            #print('bla' + production_queue[i])
            prod_queue_stats = get_template_as_dict(production_queue[i])
            #print('blu' + str(prod_queue_stats))
            GenericName = get_stat(prod_queue_stats, ["Identity", "GenericName"])            
            if GenericName is not None:
                if (GenericName == "Fauna" or "Hero" in GenericName): #================ as we don't want to have "Fauna" or "Hero", but "sheep" and e.g. "Scipio"
                    with open(save_path + civ + '/' + unit + '/ProductionQueue.txt', "a", encoding="utf-8") as f:
                        f.write("\\>" + get_stat(prod_queue_stats, ["Identity", "SpecificName"]) + "\\\\\n")
                else:
                    with open(save_path + civ + '/' + unit + '/ProductionQueue.txt', "a", encoding="utf-8") as f:
                        f.write("\\>" + get_stat(prod_queue_stats, ["Identity", "GenericName"]) + "\\\\\n")
                        
    with open(save_path + civ + '/' + unit + '/ProductionQueue.txt', "r", encoding="utf-8") as f:
        if f.read().endswith("Trains "):
            with open(save_path + civ + '/' + unit + '/ProductionQueue.txt', "w", encoding="utf-8") as f:
                f.write("%") 


#=============================================================================================================
                #========== WIP
                #==== affects

    unit_classes = get_stats(stats, ["Identity", "Classes"])
    if unit_classes is not None:
        print("This are the units classes: " + str(unit_classes))


        with open(save_path + civ + '/' + unit + '/improvements.txt', "w", encoding="utf-8") as h:
            h.write("Improvements \\>\\begin{tabular}[t]{@{}p{0.65\\textwidth}}")
        for filename in glob.glob(save_path + "Technologies/*/affects_temp.txt"):
            with open(filename) as f:
                affects_temp = f.read()
                if affects_temp in unit_classes:
                    print("HALT! " + filename)
                    bla = filename
                    bla = bla.rstrip("affects_temp.txt")
                    bla = bla + "genName.txt"
                    with open(bla) as g:
                        blu = g.read()
                        print("This is the genName: " + blu)
                        with open(save_path + civ + '/' + unit + '/improvements.txt', "a", encoding="utf-8") as h:
                            h.write(blu+ ", ")
                #==== problem e.g. CitizenSoldier Melee
                if " " in affects_temp:
                    first_part_of_long_affects_name = affects_temp[:affects_temp.find(" ")]
                    print(first_part_of_long_affects_name)
                    if affects_temp.find("\n") != -1:
                        secnd_part_of_long_affects_name = affects_temp[affects_temp.find(" ") + len(" "):affects_temp.find("\n")]
                    else:
                        secnd_part_of_long_affects_name = affects_temp[affects_temp.find(" ") + len(" "):]
                    print(secnd_part_of_long_affects_name)
                    if (first_part_of_long_affects_name in unit_classes) and (secnd_part_of_long_affects_name in unit_classes):
                        print("HALT! " + filename)
                        bla = filename
                        bla = bla.rstrip("affects_temp.txt")
                        bla = bla + "genName.txt"
                        with open(bla) as g:
                            blu = g.read()
                            print("This is the genName: " + blu)
                            with open(save_path + civ + '/' + unit + '/improvements.txt', "a", encoding="utf-8") as h:
                                h.write(blu+ ", ")



    with open(save_path + civ + '/' + unit + '/improvements.txt', "r", encoding="utf-8") as f:
        bla = f.read()
        if bla.endswith(", "):
            bla = bla.rstrip(", ")
            with open(save_path + civ + '/' + unit + '/improvements.txt', "w", encoding="utf-8") as f:
                f.write(bla) 
                    
    with open(save_path + civ + '/' + unit + '/improvements.txt', "a+", encoding="utf-8") as f:
        f.write("\\end{tabular}\\\\")                  
    with open(save_path + civ + '/' + unit + '/improvements.txt', "r", encoding="utf-8") as f:
        if f.read().endswith("Improvements \\>\\begin{tabular}[t]{@{}p{0.65\\textwidth}}\\end{tabular}\\\\"):
            with open(save_path + civ + '/' + unit + '/improvements.txt', "w", encoding="utf-8") as f:
                f.write("%") 
                    



    


        

    i = 0
    Technologies_Raw = get_stats(stats, ["ProductionQueue", "Technologies"])
    if Technologies_Raw is not None:
        for i in range(len(Technologies_Raw)):
            if Technologies_Raw[i].startswith("pair_") is True:
                print(Technologies_Raw[i])
                with open('simulation/data/technologies/' + Technologies_Raw[i] + '.json') as data_file:    
                    data = json.load(data_file)
                    if "top" in data:
                            Technologies_Raw.append(data["top"])
                    if "bottom" in data:
                            Technologies_Raw.append(data["bottom"])


                
            if not Technologies_Raw[i].startswith("pair"):
                with open('simulation/data/technologies/' + Technologies_Raw[i] + '.json') as data_file:    
                    data = json.load(data_file)

                make_sure_path_exists(save_path + "Technologies/" + Technologies_Raw[i])
                
                with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "gen_Name.txt", "w", encoding="utf-8") as f:
                    if "genericName" in data:
                        f.write(data["genericName"])
                    else:
                        f.write("%")

                with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "descript.txt", "w", encoding="utf-8") as f:
                    if "description" in data:
                        f.write("\\emph{" + data["description"] + "}")
                    else:
                        f.write("%")

                with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "require.txt", "w", encoding="utf-8") as f:
                    if "requirementsTooltip" in data:
                        bla = data["requirementsTooltip"]
                        f.write("Requirements \\>\\begin{tabular}[t]{@{}p{0.65\\textwidth}}" + bla + "\\end{tabular}\\\\")
                    else:
                        f.write("%")

                with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "tooltip.txt", "w", encoding="utf-8") as f:
                    if "tooltip" in data:
                        bla = data["tooltip"]
                        bla = bla.replace("%", "\\%")
                        f.write(bla)
                    else:
                        f.write("%")

                with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "portrait.txt", "w", encoding="utf-8") as f:
                    if "icon" in data:
                        f.write(data["icon"])
                    else:
                        f.write("%")

                if Is_there_an_Internet_connection is True:
                    if "icon" in data:
                        portrait = data["icon"]
                        PortraitFile = os.path.isfile(save_path + "Technologies/" + Technologies_Raw[i] + '/pic.png')
                        if PortraitFile is False:
                            f = urllib.request.urlopen(InternetPathForIcons + "technologies/" + portrait)
                            with open(save_path + "Technologies/" + Technologies_Raw[i] + '/pic.png', 'wb') as code:
                                code.write(f.read())


                Tech_affect_txt = os.path.isfile(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "affects_temp.txt")
                if Tech_affect_txt is True:
                    os.remove(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "affects_temp.txt")
           
                with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "affects_temp.txt", "w", encoding="utf-8") as f:
                    if "affects" in data:
                        f.write("\n".join(data["affects"]))
                    else:
                        if "modifications" in data:
                            k = 0
                            while k < len(data["modifications"]):
                                if "affects" in data["modifications"][k]:
                                    f.write("\n".join(data["modifications"][k]["affects"]) + "\n")
                                k += 1

                        if "supersedes" in data:
                            parent_technology = data["supersedes"]
                            with open('simulation/data/technologies/' + parent_technology + '.json') as data_file_parent:    
                                data_parent = json.load(data_file_parent)
                                if "affects" in data_parent:
                                    f.write("\n".join(data_parent["affects"]))
                        else:
                            f.write("%")
                

                if "researchTime" in data:
                    value_researchTime = data["researchTime"]
                else:
                    value_researchTime = "0"
                   
                if "cost" in data:
                    if "food" in data["cost"]:
                        value_food = data["cost"]["food"]
                    else:
                        value_food = "0"
                        
                    if "wood" in data["cost"]:
                        value_wood = data["cost"]["wood"]
                    else:
                        value_wood = "0"
                        
                    if "stone" in data["cost"]:
                        value_stone = data["cost"]["stone"]
                    else:
                        value_stone = "0"
                        
                    if "metal" in data["cost"]:
                        value_metal = data["cost"]["metal"]
                    else:
                        value_metal = "0"
                else:
                    value_food = "0"
                    value_wood = "0"
                    value_stone = "0"
                    value_metal = "0"

                if value_food > 0:
                    value_food = "\\includegraphics[width=0.0225\\textwidth]{food.png}\\," + str(value_food) + ",\\,"
                else:
                    value_food = ""

                if value_wood > 0:
                    value_wood = "\\includegraphics[width=0.0225\\textwidth]{wood.png}\\," + str(value_wood) + ",\\,"
                else:
                    value_wood = ""

                if value_stone > 0:
                    value_stone = "\\includegraphics[width=0.0225\\textwidth]{stone.png}\\," + str(value_stone) + ",\\,"
                else:
                    value_stone = ""

                if value_metal > 0:
                    value_metal = "\\includegraphics[width=0.0225\\textwidth]{metal.png}\\," + str(value_metal) + ",\\,"
                else:
                    value_metal = ""

                with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "cost.txt", "w", encoding="utf-8") as f:
                    if (value_researchTime == "0") and (value_food == "0") and (value_wood == "0") and (value_stone == "0") and (value_metal == "0"):
                        f.write("%")
                    else:
                        f.write( "Cost \>" + value_food + value_wood + value_stone + value_metal +'\\includegraphics[width=0.0225\\textwidth]{buildtime.png}\\,'+str(value_researchTime))

            if Keep_Tex_Files is False:
                if not Technologies_Raw[i].startswith("pair"):
                    with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "affects_temp.txt", "r", encoding="utf-8") as g:
                        bla = g.read()
                        bla = bla.replace("\n", ", ")
                        bla = bla.rstrip("%")
                        bla = bla.rstrip(", ")
                        with open(save_path + "Technologies/" + Technologies_Raw[i] + "/" + "affects.txt", "w", encoding="utf-8") as h:
                            h.write("Affects \\>\\begin{tabular}[t]{@{}p{0.65\\textwidth}}" + bla + "\\end{tabular}\\\\")
                    with open("tex-files/example_tech.tex", "r", encoding="utf-8") as f:
                        get_tex_example_tech = f.read()
                        get_tex_output_file_tech = get_tex_example_tech.replace("EXAMPLE", "Technologies/" + Technologies_Raw[i])
                    make_sure_path_exists('tex-files/Technologies/')
                    make_sure_path_exists('tex-files/' + civ + '/units/')
                    with open('tex-files/Technologies/' + Technologies_Raw[i] + '.tex', "w", encoding="utf-8") as f:
                        f.write(get_tex_output_file_tech)

        i += 1
        remove_double_entries_Technology_Raw = list(set(Technologies_Raw))
        with open(save_path + civ + '/' + unit + '/Technology_Temp.txt', "w", encoding="utf-8") as f:
            f.write("\n".join(remove_double_entries_Technology_Raw))




    



                
        
        

#=============================================================================================================
#======[18:33] <leper> Why not create a dictionary when parsing the productionqueue entities and adding the current building to the unit name?
#======[18:34] <leper> and in the end writing the result of this to each units' trainable file?
#======[18:39] <qwertz> leper: i guess that would be nicer and faster? Anyway, I'm not really sure how to do it that way, so I'll stick with the current approach at the moment :P
#======[18:39] <leper> It would at least be nicer, and not depend on file I/O that much (not that it would really matter here) ;-)
#======[18:40] <leper> qwertz: trainable_in = {}
#======[18:41] <leper> and then in the loop that writes the names currently
#======[18:42] <leper> if unit (replace this with the variable you're looping over) not in trainable_in:
#======[18:43] <leper>    trainable_in[unit] = []
#======[23:38] <leper> trainable_in[unit].extend(productionqueue_unit)
#=============================================================================================================

    with open(save_path + civ + '/' + unit + '/trains_in.txt', "w", encoding="utf-8") as f:
        f.write("Train in ")
        
    if unit.startswith("units"):
        for root,dirs,folder in os.walk(save_path):
            for files in folder:
                if files.endswith("ProductionQueue_temp.txt"):
                     with open(os.path.join(root, files), "r", encoding="utf-8") as f:
                         input_file = f.read()
                         is_this_unit_trainable = str(input_file.find(unit))
                         #==============================================
                         #======= Special handling for packed siege ====
                         #==============================================
                         if unit.endswith("unpacked"):
                             unit_siege_packed = removeSuffix(unit, "unpacked") + "packed"
                             is_this_unit_trainable = str(input_file.find(unit_siege_packed))
                         
                         if is_this_unit_trainable != str(-1):
                             with open(os.path.join(root) + '/gen_name.txt', "r", encoding="utf-8") as f:
                                 can_train_unit = str(f.read())
                                 with open(save_path + civ + '/' + unit + '/trains_in.txt', "a", encoding="utf-8") as f:
                                    f.write("\\>" + can_train_unit + "\\\\\n")

    with open(save_path + civ + '/' + unit + '/trains_in.txt', "r", encoding="utf-8") as f:
        if f.read().endswith("Train in "):
            with open(save_path + civ + '/' + unit + '/trains_in.txt', "w", encoding="utf-8") as f:
                f.write("%")







    if Keep_Tex_Files is False:
        with open("tex-files/example.tex", "r", encoding="utf-8") as f:
            get_tex_example = f.read()
            get_tex_output_file = get_tex_example.replace("EXAMPLE", civ + '/' + unit)
        make_sure_path_exists('tex-files/' + civ + '/structures/')
        make_sure_path_exists('tex-files/' + civ + '/units/')
        with open('tex-files/' + civ + '/' + unit + '.tex', "w", encoding="utf-8") as f:
            f.write(get_tex_output_file)
        
        








                
with open('simulation/data/technologies/phase_town.json') as data_file:    
    data = json.load(data_file)

print(data["genericName"])
print(data["description"])
print(data["requirementsTooltip"])
print(data["tooltip"])
print(data["icon"])
print(data["affects"])
print(data["researchTime"])
print(data["cost"]["food"])
print(data["cost"]["wood"])
print(data["cost"]["stone"])
print(data["cost"]["metal"])


 
# main
units = [
          [ "units/rome_infantry_swordsman_b",
            "units/rome_infantry_spearman_b",
            "units/rome_infantry_javelinist_b",
            "units/rome_champion_infantry",
            "units/rome_cavalry_spearman_b",
            "units/rome_champion_cavalry",
            "units/rome_hero_scipio",
            "units/rome_hero_marcellus",
            "units/rome_hero_maximus",
            "units/rome_mechanical_siege_ballista_unpacked",
            "units/rome_mechanical_siege_scorpio_unpacked",
            "units/rome_mechanical_siege_ram",  
            "units/rome_support_female_citizen",
            "units/rome_support_healer_b",
            "units/rome_support_trader",
            "units/rome_ship_fishing",
            "units/rome_ship_merchant",
            "units/rome_ship_bireme",
            "units/rome_ship_trireme",
            "units/rome_ship_quinquereme",

            "units/athen_infantry_spearman_b",
            "units/athen_infantry_javelinist_b",
            "units/athen_infantry_slinger_b",
            "units/athen_infantry_archer_b",
            "units/athen_champion_infantry",
            "units/athen_champion_marine",
            "units/athen_champion_ranged",
            "units/athen_cavalry_swordsman_b",
            "units/athen_cavalry_javelinist_b",
            "units/athen_hero_iphicrates",
            "units/athen_hero_pericles",
            "units/athen_hero_themistocles",
            "units/athen_mechanical_siege_lithobolos_unpacked",
            "units/athen_mechanical_siege_oxybeles_unpacked",
            "units/athen_support_female_citizen",
            "units/athen_support_healer_b",
            "units/athen_support_trader",
            "units/athen_ship_fishing",
            "units/athen_ship_merchant",
            "units/athen_ship_bireme",
            "units/athen_ship_trireme",

            "units/spart_infantry_spearman_b",
            "units/spart_infantry_javelinist_b",
            "units/spart_champion_infantry_sword",
            "units/spart_champion_infantry_spear",
            "units/spart_cavalry_javelinist_b",
            "units/spart_hero_agis",
            "units/spart_hero_leonidas",
            "units/spart_mechanical_siege_ram",
            "units/spart_support_female_citizen",
            "units/spart_support_healer_b",
            "units/spart_support_trader",
            "units/spart_ship_fishing",
            "units/spart_ship_merchant",
            "units/spart_ship_bireme",
            "units/spart_ship_trireme",
            
            "units/mace_infantry_spearman_b",
            "units/mace_infantry_javelinist_b",
            "units/mace_infantry_slinger_b",
            "units/mace_infantry_archer_b",
            "units/mace_champion_infantry_a",
            "units/mace_cavalry_spearman_b",
            "units/mace_cavalry_javelinist_b",
            "units/mace_champion_cavalry",
            "units/mace_hero_alexander",
            "units/mace_mechanical_siege_lithobolos_unpacked",
            "units/mace_mechanical_siege_oxybeles_unpacked",
            "units/mace_mechanical_siege_ram",
            "units/mace_mechanical_siege_tower",
            "units/mace_hero_demetrius",
            "units/mace_hero_philip",
            "units/mace_support_female_citizen",
            "units/mace_support_healer_b",
            "units/mace_support_trader",
            "units/mace_ship_fishing",
            "units/mace_ship_merchant",
            "units/mace_ship_bireme",
            "units/mace_ship_trireme",
            
            "units/cart_infantry_swordsman_b",
            "units/cart_infantry_swordsman_2_b",
            "units/cart_infantry_spearman_b",
            "units/cart_infantry_javelinist_b",
            "units/cart_infantry_slinger_b",
            "units/cart_infantry_archer_b",
            "units/cart_champion_infantry",
            "units/cart_cavalry_swordsman_b",
            "units/cart_cavalry_swordsman_2_b",
            "units/cart_cavalry_spearman_b",
            "units/cart_cavalry_javelinist_b",
            "units/cart_champion_cavalry",
            "units/cart_hero_hamilcar",
            "units/cart_hero_hannibal",
            "units/cart_hero_maharbal",
            "units/cart_mechanical_siege_ballista_unpacked",
            "units/cart_mechanical_siege_oxybeles_unpacked",
            "units/cart_support_female_citizen",
            "units/cart_support_healer_b",
            "units/cart_support_trader",
            "units/cart_ship_fishing",
            "units/cart_ship_merchant",
            "units/cart_ship_bireme",
            "units/cart_ship_trireme",
            "units/cart_ship_quinquereme",
            
            "units/brit_infantry_spearman_b",
            "units/brit_infantry_javelinist_b",
            "units/brit_infantry_slinger_b",
            "units/brit_war_dog_b",
            "units/brit_champion_infantry",
            "units/brit_cavalry_swordsman_b",
            "units/brit_cavalry_javelinist_b",
            "units/brit_champion_cavalry",
            "units/brit_hero_boudicca",
            "units/brit_hero_caratacos",
            "units/brit_hero_cynvelin",
            "units/brit_mechanical_siege_ram",
            "units/brit_support_female_citizen",
            "units/brit_support_healer_b",
            "units/brit_support_trader",
            "units/brit_ship_fishing",
            "units/brit_ship_merchant",
            "units/brit_ship_trireme",

            "units/gaul_infantry_spearman_b",
            "units/gaul_infantry_javelinist_b",
            "units/gaul_infantry_slinger_b",
            "units/gaul_champion_infantry",
            "units/gaul_cavalry_swordsman_b",
            "units/gaul_cavalry_javelinist_b",
            "units/gaul_champion_cavalry",
            "units/gaul_hero_brennus",
            "units/gaul_hero_britomartus",
            "units/gaul_hero_vercingetorix",
            "units/gaul_mechanical_siege_ram",
            "units/gaul_support_female_citizen",
            "units/gaul_support_healer_b",
            "units/gaul_support_trader",
            "units/gaul_ship_fishing",
            "units/gaul_ship_merchant",
            "units/gaul_ship_trireme",

            "units/iber_infantry_swordsman_b",
            "units/iber_infantry_spearman_b",
            "units/iber_infantry_javelinist_b",
            "units/iber_infantry_slinger_b",
            "units/iber_champion_infantry",
            "units/iber_cavalry_spearman_b",
            "units/iber_cavalry_javelinist_b",
            "units/iber_champion_cavalry",
            "units/iber_hero_caros",
            "units/iber_hero_indibil",
            "units/iber_hero_variato",
            "units/iber_mechanical_siege_ram",
            "units/iber_support_female_citizen",
            "units/iber_support_healer_b",
            "units/iber_support_trader",
            "units/iber_ship_fishing",
            "units/iber_ship_merchant",
            "units/iber_ship_fire",

            "units/pers_infantry_spearman_b",
            "units/pers_infantry_javelinist_b",
            "units/pers_infantry_archer_b",
            "units/pers_champion_infantry",
            "units/pers_cavalry_swordsman_b",
            "units/pers_cavalry_spearman_b",
            "units/pers_cavalry_javelinist_b",
            "units/pers_cavalry_archer_b",
            "units/pers_champion_cavalry",
            "units/pers_hero_cyrus",
            "units/pers_hero_darius",
            "units/pers_hero_xerxes",
            "units/pers_mechanical_siege_ram",
            "units/pers_support_female_citizen",
            "units/pers_support_healer_b",
            "units/pers_support_trader",
            "units/pers_ship_fishing",
            "units/pers_ship_merchant",
            "units/pers_ship_bireme",
            "units/pers_ship_trireme",

            "units/maur_infantry_swordsman_b",
            "units/maur_infantry_spearman_b",
            "units/maur_infantry_archer_b",
            "units/maur_champion_infantry",
            "units/maur_champion_maiden_archer",
            "units/maur_cavalry_javelinist_b",
            "units/maur_cavalry_archer_b",
            "units/maur_champion_elephant",
            "units/maur_champion_chariot",
            "units/maur_hero_ashoka",
            "units/maur_hero_chanakya",
            "units/maur_hero_maurya",
            "units/maur_support_female_citizen",
            "units/maur_support_healer_b",
            "units/maur_support_trader",
            "units/maur_support_elephant",
            "units/maur_ship_fishing",
            "units/maur_ship_merchant",
            "units/maur_ship_trireme"
                        
          ]
        ]
 
for civtemplates in units:
    for unit in civtemplates:
        printUnit(unit)
